import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.io.*;
import javax.imageio.ImageIO;

public class TileGridExample extends JPanel implements ActionListener, MouseListener, KeyListener 
{
    public static int WIDTH=800;
    public static int HEIGHT=800;
    private Font titleFont, regularFont;
    private int x;

    private Timer timer;
    private Image virtualMem;   
    private TileGrid     grid;

    private Button myButton;
    private int secs;
    private String message;
    private int keyCode;
    private String key;

    public TileGridExample() 
    {
        grid= new TileGrid(4, 4 );
        timer=new Timer(1000, this);
        virtualMem = createImage(getWidth(),getHeight());

        myButton= new Button("Start");
        //the class is its own button listener
        myButton.addActionListener (this);
        addMouseListener(this);
        this.addKeyListener(this);
        add(myButton); 
        setMessage("Click Start to begin");
        this.requestFocus();
    }

    public static void main(String[] args) {
        TileGridExample app= new TileGridExample();
        JFrame window = new JFrame("Timer Example");
        window.setSize(WIDTH, HEIGHT);
        window.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        window.getContentPane().add(app);
        window.addKeyListener(app);
        //window.pack();
        window.setVisible(true);

    }

    public void paintComponent(Graphics g){
        super.paintComponent(g);
        // We draw using the object methods of
        // the graphics buffer, not what is
        // currently on the screen
        virtualMem = createImage(getWidth(),getHeight());
        Graphics gBuffer = virtualMem.getGraphics();
        gBuffer.setColor(Color.white);
        gBuffer.fillRect(0,0,getWidth(),getHeight());
        gBuffer.setColor(Color.black);
        
        grid.draw(gBuffer, getWidth()/6, getHeight()/6, 
                getWidth()/6);
        gBuffer.drawString("Time: "+getTime(), 20,20);
        
        gBuffer.drawString(getMessage(), 20, getHeight()-20);
        
        //Now we send the result to the screen
        g.drawImage(virtualMem,0,0,this);

    }
    public void update(Graphics g)
    {
        paint(g); //get rid of flicker with this method
    }
    public int randInt(int max){
            return (int)(max*Math.random());
    }
    public boolean isTimerRunning(){
             return timer.isRunning();
    }
    public int getTime(){
            return secs;
    }
    public void setTime(int t){
            secs=t;
    }
    public void resetTimer(){
            secs=0;
            timer.stop();
    }
    public Color randomColor(){
            int red=randInt(255);
            int green=randInt(255);
            int blue=randInt(255);
            return new Color(red,green,blue);
    }
    public boolean isTimerEvent(ActionEvent e){
            Object source = e.getSource();
            return source == timer;
    }
    public void actionPerformed(ActionEvent e) {
        
        if ( isTimerEvent(e)) {
            setTime(getTime()+1);           
        }else{
            //restart button pressed
              
              if (isTimerRunning()) {
                  stopTimer();
                  
                  setMessage("Click Start to begin a New Puzzle");
               }else{
                   resetTimer();
                   startTimer();
                   grid= new TileGrid(4, 4 );
                   
                   setMessage("Click Grid to move Tiles");
               }
        }
        this.requestFocus();
        repaint();
    }
    public void startTimer() {
        timer.start();
        myButton.setLabel("Stop");
        
    }
    public void stopTimer() {
        timer.stop();
        myButton.setLabel("Start");
    }
    public String getMessage(){
        return message;
    }
    public void setMessage(String message){
        this.message=message;
    }
    // implement the MouseListener
    public void mouseClicked(MouseEvent e) {}
    public void mouseEntered(MouseEvent e) {}
    public void mouseExited(MouseEvent e) {}
    public void mousePressed(MouseEvent e) {}
    public void mouseReleased(MouseEvent e) {
        this.requestFocus();
        if (isTimerRunning()){
            int x = e.getX();
            int y = e.getY();
            int r = grid.getRow(y);
            int c = grid.getCol(x);
            setMessage("Clicked "+grid.getRowColString(x, y));
            if (grid.isEmpty(r, c)){
                // img can be any size, place in same folder as class files
                // File f=new File("Ship.gif");
                // BufferedImage img=null;
                // try {
                //     img = ImageIO.read(f);
                // } catch (IOException err) {
                //     err.printStackTrace();
                // }
                // Tile t=new Tile(img);
                // or else default is random colored disk
                Tile t=new Tile(randomColor());
                grid.add(t, r, c);
                setMessage(getMessage()+" New Tile Added");
            }else{
                setMessage(getMessage()+" Not a Valid Move");
            }
        }else{
            setMessage("Hummm...Try pressing the Start Button First");
        }
        repaint();
    }
    //KeyListener Methods
    public void keyPressed(KeyEvent e) {
        keyCode=e.getKeyCode();
        key=KeyEvent.getKeyText(keyCode);
        message="You pressed code "+keyCode+" \""+key+"\"";
        repaint();
    }
    public void keyReleased(KeyEvent e) {}
    public void keyTyped(KeyEvent e) {}

}